/**
Cchar: Class holding the character information.
**/
#include "CStats.h"

#ifndef _CCHAR_H_
#define _CCHAR_H_

#define FLAG_GM		0x80000000
#define FLAG_INVITE 0x00000010
#define FLAG_ANON	0x00000020

class CChar {

public:

	unsigned int	acctid;
	unsigned int	charid;
	char			charname[16];
	unsigned short	targid;
	unsigned int	ip;
	unsigned int	servip;
	int	weapondelay;
	bool inBattle;

	// 2008/09/04 UԊuݒ Add-Start
	// UԊuݒl
	unsigned int	atkdelay;
	// 2008/09/04 UԊuݒ Add-End
	// 2008/09/04 ړxݒ Add-Start
	// ړxݒl
	unsigned char	charspd;
	// 2008/09/04 ړxݒ Add-End

	// Chardetails
	bool locked;

	bool commListLocked;
	unsigned char firstlogin;

	// only needed at runtime, therefor not in DB.
	unsigned short currTarg;
	unsigned short movCount;

	// last active time
	int aTime;

	bool zoning;
	unsigned long long oldAttackTime;

	struct location loc;
	struct position hp;
	struct blowfish enc;
	struct jobs j;
	struct look l;
	struct equip e;
	struct skills s;
	CStats st;
	struct keys k;
	struct health h;
	struct inventory inv;

	//FAME VARIABLES

	unsigned short fameBas;
	unsigned short fameSan;
	unsigned short fameWin;
	unsigned short fameNorg;

	// list of all commands to be executed for this char
	std::vector<CComm> commList;

	// list of all npcs/ to be executed for this char
	std::vector<int> spawnIDList;

	// pointer to next and previous char in list
	CChar * prev, * next;

	// global object (start of charlist)
	static CChar first;

	// std Constructor for CChar
	CChar::CChar();

	// remove char from charlist
	void CChar::remove();

	// add char to list
	void CChar::insert(CChar schar);

	// load a char
	int CChar::loadChar(int charid);

	// creates a new character in the DB
	int CChar::createChar();

	// write char to DB
	int CChar::writeChar();

	// return a pointer to the char with matching charid or charname 
	static CChar * CChar::getCChar(int charid, char charname[16],int targid);

	// is a char in the specefied zone?
	static bool CChar::isCharInZone(unsigned char zone);

	// add a command to all chars in list with the matching condition
	void CChar::insertComm(int cond, CComm comm);

	// update the chars position and write it to the DB
	void CChar::updatePos(union PacketData pd);

	// send a tell
	int CChar::sendTell(unsigned char * buf, union PacketData pd, unsigned short num);

	// send std chat and intercept commands
	int CChar::sendSay(unsigned char * data, union PacketData pd, unsigned short num);

	// set the chars position
	int CChar::setPosition(unsigned char * data, float x, float y, float z, unsigned short num);

	// que a command for  @command forced zoning
	int CChar::zoneChar(unsigned char to);

	// request to zone
	int CChar::chip(unsigned char *pd, unsigned short num);

	// prepare char for zoning
	int CChar::changeZone();

	// actually zone
	int CChar::zoner(unsigned char * data, unsigned short num);

	// zoneing using a zoneline
	int CChar::zoneLine(union PacketData pd);

	// process @commands for this char
	int CChar::atCommand(unsigned char * buf, char * data, unsigned short num);

	// logout 
	int CChar::logout(unsigned char * data, unsigned short num);

	// allow logout
	int CChar::allowLogout(unsigned char * data, unsigned short num);

	// releases a char from frozen screen
	int CChar::release(unsigned short * data, unsigned short num);

	// change and display equipment
	int CChar::equip(unsigned char * data, char * in, unsigned short num);

	// loads a chars inventory from the db
	int CChar::loadInv(unsigned char * data, unsigned short num);

	// adds an item to the char
	int CChar::addItem(unsigned short item, unsigned char location, unsigned char quantity);

	// update the chars health status
	int CChar::health(unsigned char * data, unsigned short num);

	// change the current job and calculate the stats
	int CChar::cmjob(unsigned char * data, unsigned char job, unsigned char level, unsigned short num);

	// a debugmessage printer... it's buggy btw... heh...
	void CChar::debugMsg(char * buf, ...);

	// stops the "Downloading Data..." msg
	int CChar::stopDld(unsigned char * data, unsigned short num);;

	// drops an item from inventory.
	int CChar::dropItem(unsigned char * data, char * in, unsigned short num);

	// Sorts an inventory location
	int CChar::sortInv(unsigned char * data, char * in, unsigned short num);

	// moves items from storage to inventory and vice-versa
	int CChar::moveItem(unsigned char * data, char * in, unsigned short num);

	// change flags (/inv /anon and the like) still buggy.
	int CChar::updateChar(unsigned char * data, unsigned short num);

	// Spawn other players
	int CChar::spawnPCs(unsigned char * data, unsigned short num);

	// recieve chat
	int CChar::recvChat(unsigned char * data, unsigned short num, CComm * comm);

	// send an error msg to the char
	int CChar::sendErr(unsigned char * data, unsigned short num, unsigned short n);

	// checking of players and monsters. monsters still missing.
	int CChar::check(unsigned char * data, char * in, unsigned short num);

	// recieving of standart FFXI messages
	int CChar::stdmRecv(unsigned char * data, CComm * comm, unsigned short num);

	// sending of an emote action
	int CChar::emote(unsigned char * data, char * in, unsigned short num);

	// recieving of an emote action
	int CChar::emoteRecv(unsigned char * data, CComm * comm, unsigned short num);

	// response the the server message request
	int CChar::servmesResp(unsigned char * data, unsigned short num);

	// NOT USED ATM
	int CChar::updatePCs(unsigned char * data, CComm * comm, unsigned short num);

	// /random action
	int CChar::diceroll(unsigned char * data, unsigned short num);

	// animation change to sit.
	int CChar::tsit(unsigned char * data, char * in, unsigned short num);

	// invite, anon flag switch. still buggy.
	int CChar::seekAnon(unsigned char * data, char * in, unsigned short num);

	// sends inventory, moghouse and locker size to client.
	int CChar::updateInvSize(unsigned char * data,  unsigned short num);

	// changes job while in moghouse.
	int CChar::cmJobMog(unsigned char * data,  char * in, unsigned short num);

	// response to the 16c (character request packet)
	int CChar::reqChar(unsigned char * data,  char * in, unsigned short num);
	
	int CChar::updateKeyItems(unsigned char * data, unsigned short num);

	// Add items to commList
	int CChar::addCommList(CComm comm);

	bool CChar::hasKeyItem(char keyItemId);
	bool CChar::seenKeyItem(char keyItemId);

	//Calculate your weapon delay
	void CChar::calculateAttackSpeed();

};


#endif